import os
import os.path as osp

from setuptools import find_packages
from typing import List

from pygments.formatters import HtmlFormatter
from pygments.styles import get_style_by_name
from pygments.token import Text
from pygments import highlight


def find_all_themes_packages() -> List[ str ]:
    """Finds the current supported themes in the a11y pygments package."""
    exclude = { 'test', 'a11y_pygments', 'a11y_pygments.utils' }
    packages = set( find_packages() )
    themes = list( packages - exclude )
    themes = [ x.split( '.' )[ 1 ] for x in themes ]
    return themes


def find_all_themes() -> List[ str ]:
    """Finds the current supported themes names in the a11y pygments package."""
    exclude = { 'test', 'a11y_pygments', 'a11y_pygments.utils' }
    packages = set( find_packages() )
    themes = list( packages - exclude )
    themes = [ x.split( '.' )[ 1 ] for x in themes ]
    themes = [ x.replace('_', '-') for x in themes ]
    return themes


def generate_css(themes: List[ str ], save_dir = ''):
    """Generate css for the given themes."""
    basedir = 'a11y_pygments'
    for theme in themes:
        style = get_style_by_name( theme )
        formatter = HtmlFormatter( style=style, full=True, hl_lines=[2,3,4] )
        css = formatter.get_style_defs()
        color = style.style_for_token(Text)['color']
        css += "\n .highlight { background: %s; color: #%s; }"%(
            style.background_color, color
        )
        package = theme.replace('-', '_')
        out = osp.join( basedir, package, 'style.css' )
        with open( out, 'w' ) as f:
            f.write( css )

        if save_dir:
            if not osp.exists(osp.join(save_dir, 'css')):
                os.mkdir(osp.join(save_dir, 'css'))
            out = osp.join(save_dir, 'css', package + '-style.css')
            with open( out, 'w' ) as f:
                f.write( css )



import ast
import os
from setuptools import setup, find_packages

from a11y_pygments.utils.utils import find_all_themes_packages


def find_entrypoints():
    themes = find_all_themes_packages()
    entrypoints = []
    base_package = 'a11y_pygments'
    for theme in themes:
        name = theme.replace('_', '-')
        package = '{}.{}.style:Theme'.format(base_package, theme)
        entrypoints.append('{} = {}'.format(name, package))
    print(entrypoints)
    return entrypoints


def get_long_description():
    with open(os.path.join(
        os.path.dirname(os.path.abspath(__file__)), 'README.md'
    ), encoding='utf8') as fp:
        return fp.read()


def get_version(module='a11y_pygments'):
    """Get version."""
    dir = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(dir, module, '__init__.py')
    with open(path, 'r') as f:
        data = f.read()
    lines = data.split('\n')
    for line in lines:
        if line.startswith('VERSION_INFO'):
            version_tuple = ast.literal_eval(line.split('=')[-1].strip())
            version = '.'.join(map(str, version_tuple))
            break
    return version


setup (
  name='accessible-pygments',
  version=get_version(),
  description='A collection of accessible pygments styles',
  long_description=get_long_description(),
  long_description_content_type='text/markdown',
  license='BSD',
  keywords='pygments style accessible a11',

  author='Stephannie Jimenez Gacha',
  author_email='steff456@hotmail.com',

  url='https://github.com/Quansight-Labs/accessible-pygments',

  packages=find_packages(exclude=['test']),
  install_requires=[
    'pygments >= 1.5'
  ],

  entry_points ={
    "pygments.styles": find_entrypoints()
  },

  classifiers=[
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
    'Topic :: Software Development :: Libraries :: Python Modules',
  ]
)



from pygments.style import Style
from pygments.token import (
    Keyword, Name, Comment, String, Error, Text, Number, Operator, Generic,
    Punctuation, Other, Literal)


class Colors:
    comment = "#F5F5F5"
    red = "#AB6369"
    orange = "#B89784"
    yellow = "#CAAB6D"
    green = "#81B19B"
    blue = "#6F98B3"
    purple = "#b19db4"
    black = "#F5F5F5"


class Theme(Style):
    """
    This style mimics the gotthard dark theme from vscode.
    """

    default_style = ''

    background_color = "#000000"
    highlight_color = "#4c4b4be8"

    styles = {
        Text:                      Colors.black,    # class:  ''
        Error:                     Colors.red,      # class: 'err'
        Other:                     "",              # class 'x'

        Comment:                   Colors.purple,   # class: 'c'

        Keyword:                   Colors.purple,   # class: 'k'
        Keyword.Constant:          Colors.red,      # class: 'kc'
        # Keyword.Declaration:       "",            # class: 'kd'
        # Keyword.Namespace:         "",            # class: 'kn'
        # Keyword.Pseudo:            "",            # class: 'kp'
        # Keyword.Reserved:          "",            # class: 'kr'
        Keyword.Type:              Colors.green,    # class: 'kt'

        Operator:                  Colors.blue,     # class: 'o'
        Operator.Word:             Colors.purple,   # class: 'ow'

        Punctuation:               Colors.black,    # class: 'p'

        Name:                      Colors.black,    # class: 'n'
        Name.Attribute:            Colors.purple,   # class: 'na'
        Name.Builtin:              Colors.green,    # class: 'nb'
        Name.Builtin.Pseudo:       Colors.green,    # class: 'bp'
        Name.Class:                Colors.yellow,   # class: 'nc'
        Name.Constant:             Colors.red,      # class: 'no'
        Name.Decorator:            Colors.green,    # class: 'nd'
        Name.Entity:               Colors.green,    # class: 'ni'
        Name.Exception:            Colors.red,      # class: 'ne'
        Name.Function:             Colors.purple,   # class: 'nf'
        Name.Property:             Colors.purple,   # class: 'py'
        Name.Label:                Colors.green,    # class: 'nl'
        Name.Namespace:            Colors.yellow,   # class: 'nn'
        # Name.Other:                "",            # class: 'nx'
        Name.Tag:                  Colors.red,      # class: 'nt'
        Name.Variable:             Colors.comment,  # class: 'nv'
        Name.Variable.Magic:       Colors.comment,
        # Name.Variable.Class:       "",            # class: 'vc'
        # Name.Variable.Global:      "",            # class: 'vg'
        # Name.Variable.Instance:    "",            # class: 'vi'

        Number:                    Colors.red,      # class: 'm'
        # Number.Float:              "",            # class: 'mf'
        # Number.Hex:                "",            # class: 'mh'
        # Number.Integer:            "",            # class: 'mi'
        # Number.Integer.Long:       "",            # class: 'il'
        # Number.Oct:                "",            # class: 'mo'

        Literal:                   Colors.purple,   # class: 'l'
        # Literal.Date:              "",            # class: 'ld'

        String:                    Colors.green,    # class: 's'
        String.Backtick:           Colors.yellow,   # class: 'sb'
        # String.Char:               "",            # class: 'sc'
        # String.Doc:                "",            # class: 'sd'
        # String.Double:             "",            # class: 's2'
        String.Escape:             Colors.blue,     # class: 'se'
        # String.Heredoc:            "",            # class: 'sh'
        # String.Interpol:           "",            # class: 'si'
        # String.Other:              "",            # class: 'sx'
        String.Regex:              Colors.blue,     # class: 'sr'
        # String.Single:             "",            # class: 's1'
        String.Symbol:             Colors.green,    # class: 'ss'

        # Generic:                   "",            # class: 'g'
        Generic.Deleted:           Colors.red,      # class: 'gd',
        # Generic.Emph:              "italic",      # class: 'ge'
        # Generic.Error:             "",            # class: 'gr'
        Generic.Heading:           Colors.green,    # class: 'gh'
        Generic.Subheading:        Colors.green,    # class: 'gu'
        # Generic.Inserted:          "",            # class: 'gi'
        # Generic.Output:            "",            # class: 'go'
        # Generic.Prompt:            "",            # class: 'gp'
        Generic.Strong:            "bold",          # class: 'gs'
        # Generic.Traceback:         "",            # class: 'gt'
    }